//Douglas Fjällrud, Axel Blomén
package src.rsa;

import java.math.BigInteger;

public class EuclideanAlgorithm {

    public static BigInteger modularInverse(BigInteger a, BigInteger m) {
        BigInteger m0 = m, t, q;
        BigInteger x0 = BigInteger.ZERO, x1 = BigInteger.ONE;
    
        if (m.equals(BigInteger.ONE)) return BigInteger.ZERO;
    
        while (a.compareTo(BigInteger.ONE) > 0) {
            q = a.divide(m);  
            t = m;
    
            m = a.mod(m);
            a = t;
            t = x0;
    
            x0 = x1.subtract(q.multiply(x0));
            x1 = t;
        }
    
        if (x1.compareTo(BigInteger.ZERO) < 0) {
            x1 = x1.add(m0);
        }
    
        return x1;
    }

    public static BigInteger gcd(BigInteger a, BigInteger b) {
        if (b.equals(BigInteger.ZERO)) {
            return a;
        } else {
            return gcd(b, a.mod(b));
        }
    }

    public static void main(String[] args) {
        System.out.println("=== Test av modular invers ===");
    
        BigInteger a = new BigInteger("65537"); 
        BigInteger m = new BigInteger("973157123091");
    
        if (gcd(a, m).equals(BigInteger.ONE)) {
            BigInteger inv = modularInverse(a, m);
            System.out.println("Modulär invers av " + a + " mod " + m + " är: " + inv);
            System.out.println("Verifiering: (" + a + " * " + inv + ") mod " + m + " = " + a.multiply(inv).mod(m));
        } else {
            System.out.println("Ingen invers existerar för a mod m.");
        }
    }
}
